@@ -1,5 +1,6 @@
 from pathlib import Path
 import sys
+import re
 import platform
 import yaml
 import json
@@ -7,7 +8,7 @@ from yaml.parser import ParserError
 import argparse
 import logging
 import os
-from typing import Callable, Iterable, List
+from typing import Callable, List
 from spectacles import __version__
 from spectacles.runner import Runner
 from spectacles.client import LookerClient
@@ -199,6 +200,15 @@ def handle_exceptions(function: Callable) -> Callable:
     return wrapper
 
 
+def preprocess_dashes(args: List[str]) -> List[str]:
+    """Replace any dashes with tildes, otherwise argparse will assume they're options"""
+    pattern = re.compile(r"^-(?=([\w_\*]+/[\w_\*]+)|(\d+)$)")
+    processed = []
+    for arg in args:
+        processed.append(pattern.sub("~", arg))
+    return processed
+
+
 @handle_exceptions
 def main():
     """Runs main function. This is the entry point."""
@@ -208,15 +218,28 @@ def main():
             title="Spectacles requires Python 3.7 or higher.",
             detail="The current Python version is %s." % platform.python_version(),
         )
+
+    args = preprocess_dashes(sys.argv[1:])
     parser = create_parser()
-    args = parser.parse_args()
+    args = parser.parse_args(args)
+
+    branch = getattr(args, "branch", None)
+    commit_ref = getattr(args, "commit_ref", None)
+    ref = branch or commit_ref
+    target = getattr(args, "target", None)
+    incremental = getattr(args, "incremental", None)
 
     # Normally would be cleaner to handle this with an argparse mutually exclusive
     # group, but this doesn't work with --commit-ref and --remote-reset also needing
     # to be mutually exclusive, so raise the error manually.
-    if getattr(args, "branch", None) and getattr(args, "commit_ref", None):
+    if branch and commit_ref:
         parser.error("argument --commit-ref not allowed with argument --branch")
 
+    if target and not incremental:
+        parser.error(
+            "argument --target can only be passed in incremental mode (--incremental)"
+        )
+
     for handler in logger.handlers:
         handler.setLevel(args.log_level)
 
@@ -241,50 +264,61 @@ def main():
         run_sql(
             args.log_dir,
             args.project,
-            args.branch,
+            ref,
             args.explores,
-            args.exclude,
             args.base_url,
             args.client_id,
             args.client_secret,
             args.port,
             args.api_version,
-            args.mode,
+            args.fail_fast,
+            args.incremental,
+            args.target,
             args.remote_reset,
             args.concurrency,
-            args.commit_ref,
             args.profile,
             args.runtime_threshold,
+            args.chunk_size,
         )
     elif args.command == "assert":
         run_assert(
             args.project,
-            args.branch,
+            ref,
             args.explores,
-            args.exclude,
             args.base_url,
             args.client_id,
             args.client_secret,
             args.port,
             args.api_version,
             args.remote_reset,
-            args.commit_ref,
         )
     elif args.command == "content":
         run_content(
             args.project,
-            args.branch,
+            ref,
             args.explores,
-            args.exclude,
             args.base_url,
             args.client_id,
             args.client_secret,
             args.port,
             args.api_version,
             args.remote_reset,
-            args.commit_ref,
             args.incremental,
+            args.target,
             args.exclude_personal,
+            args.folders,
+        )
+    elif args.command == "lookml":
+        run_lookml(
+            args.project,
+            ref,
+            args.base_url,
+            args.client_id,
+            args.client_secret,
+            args.port,
+            args.api_version,
+            args.remote_reset,
+            args.severity,
         )
 
     if not args.do_not_track:
@@ -310,6 +344,7 @@ def create_parser() -> argparse.ArgumentParser:
     )
     base_subparser = _build_base_subparser()
     _build_connect_subparser(subparser_action, base_subparser)
+    _build_lookml_subparser(subparser_action, base_subparser)
     _build_sql_subparser(subparser_action, base_subparser)
     _build_assert_subparser(subparser_action, base_subparser)
     _build_content_subparser(subparser_action, base_subparser)
@@ -418,13 +453,12 @@ def _build_validator_subparser(
     subparser_action: argparse._SubParsersAction,
     base_subparser: argparse.ArgumentParser,
 ) -> argparse.ArgumentParser:
-    """Returns the base subparser with arguments required for every validator   .
+    """Returns the base subparser with arguments required for every validator.
 
     Returns:
         argparse.ArgumentParser: validator subparser with project, branch, remote reset and import projects arguments.
 
     """
-
     base_subparser.add_argument(
         "--project",
         action=EnvVarAction,
@@ -438,24 +472,6 @@ def _build_validator_subparser(
         env_var="LOOKER_GIT_BRANCH",
         help="The branch of your project that Spectacles will use to run queries.",
     )
-    base_subparser.add_argument(
-        "--explores",
-        nargs="+",
-        default=["*/*"],
-        help="Specify the explores Spectacles should test. \
-            List of strings in 'model_name/explore_name' format. \
-            The '*' wildcard selects all models or explores. For instance,\
-            'model_name/*' would select all explores in the 'model_name' model.",
-    )
-    base_subparser.add_argument(
-        "--exclude",
-        nargs="+",
-        default=[],
-        help="Specify the explores Spectacles should exclude when testing. \
-            List of strings in 'model_name/explore_name' format. \
-            The '*' wildcard excludes all models or explores. For instance,\
-            'model_name/*' would select all explores in the 'model_name' model.",
-    )
     group = base_subparser.add_mutually_exclusive_group()
     group.add_argument(
         "--remote-reset",
@@ -473,10 +489,58 @@ def _build_validator_subparser(
             In order to test a specific commit, Spectacles will create a new branch \
             for the tests and then delete the branch when it is finished.",
     )
-
     return base_subparser
 
 
+def _build_select_subparser(
+    subparser_action: argparse._SubParsersAction,
+    base_subparser: argparse.ArgumentParser,
+) -> argparse.ArgumentParser:
+    base_subparser.add_argument(
+        "--explores",
+        nargs="+",
+        default=["*/*"],
+        help="Specify the explores Spectacles should test. \
+            List of strings in 'model_name/explore_name' format. \
+            The '*' wildcard selects all models or explores. For instance,\
+            'model_name/*' would select all explores in the 'model_name' model.",
+    )
+    return base_subparser
+
+
+def _build_lookml_subparser(
+    subparser_action: argparse._SubParsersAction,
+    base_subparser: argparse.ArgumentParser,
+) -> None:
+    """Returns the subparser for the subcommand `lookml`.
+
+    Args:
+        subparser_action (type): Description of parameter `subparser_action`.
+        base_subparser (type): Description of parameter `base_subparser`.
+
+    Returns:
+        type: Description of returned object.
+
+    """
+    subparser = subparser_action.add_parser(
+        "lookml",
+        parents=[base_subparser],
+        help="Test for LookML syntax errors.",
+    )
+    subparser.add_argument(
+        "--severity",
+        choices=["info", "warning", "error"],
+        default="warning",
+        help=(
+            "Specify a level of validation error severity to trigger test failure. "
+            "Spectacles will display all errors, regardless of severity, "
+            "but only errors at or higher than this level will cause this "
+            "validator to fail. The default is 'warning'."
+        ),
+    )
+    _build_validator_subparser(subparser_action, subparser)
+
+
 def _build_sql_subparser(
     subparser_action: argparse._SubParsersAction,
     base_subparser: argparse.ArgumentParser,
@@ -494,20 +558,32 @@ def _build_sql_subparser(
     subparser = subparser_action.add_parser(
         "sql",
         parents=[base_subparser],
-        help="Build and run queries to test your Looker instance.",
+        help="Run SQL queries to test your Looker instance.",
+    )
+    group = subparser.add_mutually_exclusive_group()
+    group.add_argument(
+        "--fail-fast",
+        action="store_true",
+        help=(
+            "Test explore-by-explore instead of dimension-by-dimension. "
+            "This means that validation takes less time but only returns the first "
+            "error identified in each explore. "
+        ),
+    )
+    group.add_argument(
+        "--incremental",
+        action="store_true",
+        help=(
+            "Only display errors which are not present on the target branch or commit. "
+            "If --target is not specified, Spectacles compares to production."
+        ),
     )
-
-    _build_validator_subparser(subparser_action, subparser)
-
     subparser.add_argument(
-        "--mode",
-        choices=["batch", "single", "hybrid"],
-        default="batch",
-        help="Specify the mode the SQL validator should run.\
-            In single-dimension mode, the SQL validator will run one query \
-            per dimension. In batch mode, the SQL validator will create one \
-            query per explore. In hybrid mode, the SQL validator will first run in \
-            batch mode and then run errored explores in single-dimension mode.",
+        "--target",
+        help=(
+            "The branch name or commit SHA to compare to for incremental testing. "
+            "Must be used with --incremental."
+        ),
     )
     subparser.add_argument(
         "--concurrency",
@@ -534,6 +610,14 @@ def _build_sql_subparser(
             "seconds."
         ),
     )
+    subparser.add_argument(
+        "--chunk-size",
+        type=int,
+        default=500,
+        help="Limit the size of explore-level queries by this number of dimensions.",
+    )
+    _build_validator_subparser(subparser_action, subparser)
+    _build_select_subparser(subparser_action, subparser)
 
 
 def _build_assert_subparser(
@@ -553,8 +637,8 @@ def _build_assert_subparser(
     subparser = subparser_action.add_parser(
         "assert", parents=[base_subparser], help="Run Looker data tests."
     )
-
     _build_validator_subparser(subparser_action, subparser)
+    _build_select_subparser(subparser_action, subparser)
 
 
 def _build_content_subparser(
@@ -564,20 +648,40 @@ def _build_content_subparser(
     subparser = subparser_action.add_parser(
         "content", parents=[base_subparser], help="Run Looker content validation."
     )
-
     subparser.add_argument(
         "--incremental",
         action="store_true",
-        help="Only display errors which are not present on the production branch.",
+        help=(
+            "Only display errors which are not present on the target branch or commit. "
+            "If --target is not specified, Spectacles compares to production."
+        ),
+    )
+    subparser.add_argument(
+        "--target",
+        help=(
+            "The branch name or commit SHA to compare to for incremental testing. "
+            "Must be used with --incremental."
+        ),
     )
-
     subparser.add_argument(
         "--exclude-personal",
         action="store_true",
         help="Exclude errors found in content in personal folders.",
     )
-
+    subparser.add_argument(
+        "--folders",
+        nargs="+",
+        help=(
+            "Specify the content folder IDs that Spectacles should test. "
+            "Spectacles will also test all content "
+            "found in subfolders of the specified folders. "
+            "Appending '-' to a folder ID will exclude it and all subfolders. "
+            "Takes precedence over --exclude-personal."
+        ),
+        default=[],
+    )
     _build_validator_subparser(subparser_action, subparser)
+    _build_select_subparser(subparser_action, subparser)
 
 
 def run_connect(
@@ -588,31 +692,79 @@ def run_connect(
 
 
 @log_duration
-def run_content(
+def run_lookml(
     project,
-    branch,
-    explores,
-    excludes,
+    ref,
     base_url,
     client_id,
     client_secret,
     port,
     api_version,
     remote_reset,
-    commit_ref,
-    incremental,
-    exclude_personal,
+    severity,
 ) -> None:
-    runner = Runner(
-        base_url, project, client_id, client_secret, port, api_version, remote_reset
+    client = LookerClient(base_url, client_id, client_secret, port, api_version)
+    runner = Runner(client, project, remote_reset)
+    results = runner.validate_lookml(ref, severity)
+    errors = sorted(results["errors"], key=lambda x: x["metadata"]["file_path"] or "a")
+    unique_files = sorted(
+        set(
+            error["metadata"]["file_path"]
+            for error in errors
+            if error["metadata"]["file_path"]
+        )
     )
+
+    for file_path in unique_files:
+        printer.print_validation_result(status="failed", source=file_path)
+
+    if errors:
+        for error in errors:
+            printer.print_lookml_error(
+                error["metadata"]["file_path"],
+                error["metadata"]["line_number"],
+                error["metadata"]["severity"],
+                error["message"],
+                error["metadata"]["lookml_url"],
+            )
+        logger.info("")
+        if results["status"] == "failed":
+            raise GenericValidationError
+    else:
+        printer.print_lookml_success()
+        logger.info("")
+
+
+@log_duration
+def run_content(
+    project,
+    ref,
+    filters,
+    base_url,
+    client_id,
+    client_secret,
+    port,
+    api_version,
+    remote_reset,
+    incremental,
+    target,
+    exclude_personal,
+    folders,
+) -> None:
+    client = LookerClient(base_url, client_id, client_secret, port, api_version)
+    runner = Runner(client, project, remote_reset)
     results = runner.validate_content(
-        branch, commit_ref, explores, excludes, incremental, exclude_personal
+        ref,
+        filters,
+        incremental,
+        target,
+        exclude_personal,
+        folders,
     )
 
     for test in sorted(results["tested"], key=lambda x: (x["model"], x["explore"])):
         message = f"{test['model']}.{test['explore']}"
-        printer.print_validation_result(passed=test["passed"], source=message)
+        printer.print_validation_result(status=test["status"], source=message)
 
     errors = sorted(
         results["errors"],
@@ -625,6 +777,8 @@ def run_content(
                 error["explore"],
                 error["message"],
                 error["metadata"]["content_type"],
+                error["metadata"].get("tile_type"),
+                error["metadata"].get("tile_title"),
                 error["metadata"]["space"],
                 error["metadata"]["title"],
                 error["metadata"]["url"],
@@ -638,25 +792,23 @@ def run_content(
 @log_duration
 def run_assert(
     project,
-    branch,
-    explores,
-    exclude,
+    ref,
+    filters,
     base_url,
     client_id,
     client_secret,
     port,
     api_version,
     remote_reset,
-    commit_ref,
 ) -> None:
-    runner = Runner(
-        base_url, project, client_id, client_secret, port, api_version, remote_reset
-    )
-    results = runner.validate_data_tests(branch, commit_ref, explores, exclude)
+    client = LookerClient(base_url, client_id, client_secret, port, api_version)
+    runner = Runner(client, project, remote_reset)
+
+    results = runner.validate_data_tests(ref, filters)
 
     for test in sorted(results["tested"], key=lambda x: (x["model"], x["explore"])):
         message = f"{test['model']}.{test['explore']}"
-        printer.print_validation_result(passed=test["passed"], source=message)
+        printer.print_validation_result(status=test["status"], source=message)
 
     errors = sorted(
         results["errors"],
@@ -681,45 +833,41 @@ def run_assert(
 def run_sql(
     log_dir,
     project,
-    branch,
-    explores,
-    exclude,
+    ref,
+    filters,
     base_url,
     client_id,
     client_secret,
     port,
     api_version,
-    mode,
+    fail_fast,
+    incremental,
+    target,
     remote_reset,
     concurrency,
-    commit_ref,
     profile,
     runtime_threshold,
+    chunk_size,
 ) -> None:
     """Runs and validates the SQL for each selected LookML dimension."""
-    runner = Runner(
-        base_url, project, client_id, client_secret, port, api_version, remote_reset
-    )
-
-    def iter_errors(lookml: List) -> Iterable:
-        for item in lookml:
-            if item.errored:
-                yield item
+    client = LookerClient(base_url, client_id, client_secret, port, api_version)
+    runner = Runner(client, project, remote_reset)
 
     results = runner.validate_sql(
-        branch,
-        commit_ref,
-        explores,
-        exclude,
-        mode,
+        ref,
+        filters,
+        fail_fast,
+        incremental,
+        target,
         concurrency,
         profile,
         runtime_threshold,
+        chunk_size,
     )
 
     for test in sorted(results["tested"], key=lambda x: (x["model"], x["explore"])):
         message = f"{test['model']}.{test['explore']}"
-        printer.print_validation_result(passed=test["passed"], source=message)
+        printer.print_validation_result(status=test["status"], source=message)
 
     errors = sorted(
         results["errors"],
@@ -737,14 +885,12 @@ def run_sql(
                 dimension=error["metadata"].get("dimension"),
                 lookml_url=error["metadata"].get("lookml_url"),
             )
-        if mode == "batch":
+        if fail_fast:
             logger.info(
                 printer.dim(
                     "\n\nTo determine the exact dimensions responsible for "
                     f"{'this error' if len(errors) == 1 else 'these errors'}, "
-                    "you can re-run \nSpectacles in single-dimension mode, "
-                    "with `--mode single`.\n\nYou can also run this original "
-                    "validation with `--mode hybrid` to do this automatically."
+                    "you can rerun \nSpectacles without --fail-fast."
                 )
             )
 
